@@ -4,10 +4,17 @@ use Imager;
 use vars qw($VERSION @ISA);
 
 BEGIN {
-  $VERSION = "0.88";
+  $VERSION = "0.78";
 
-  require XSLoader;
-  XSLoader::load('Imager::File::PNG', $VERSION);
+  eval {
+    require XSLoader;
+    XSLoader::load('Imager::File::PNG', $VERSION);
+    1;
+  } or do {
+    require DynaLoader;
+    push @ISA, 'DynaLoader';
+    bootstrap Imager::File::PNG $VERSION;
+  };
 }
 
 Imager->register_reader
@@ -16,10 +23,7 @@ Imager->register_reader
    single => 
    sub { 
      my ($im, $io, %hsh) = @_;
-     my $flags = 0;
-     $hsh{png_ignore_benign_errors}
-       and $flags |= IMPNG_READ_IGNORE_BENIGN_ERRORS;
-     $im->{IMG} = i_readpng_wiol($io, $flags);
+     $im->{IMG} = i_readpng_wiol($io);
 
      unless ($im->{IMG}) {
        $im->_set_error(Imager->_error_as_msg);
@@ -70,7 +74,7 @@ Imager's PNG support is documented in L<Imager::Files>.
 
 =head1 AUTHOR
 
-Tony Cook <tonyc@cpan.org>
+Tony Cook <tony@imager.perl.org>
 
 =head1 SEE ALSO
 
